# setup tests with all doctests found in docs/

from plone.app.customerize import docs
from plone.app.customerize.tests import layer
from Testing.ZopeTestCase import FunctionalDocFileSuite
from Products.PloneTestCase import PloneTestCase
from Products.Five.testbrowser import Browser
from unittest import TestSuite
from os.path import join, split, abspath, dirname
from os import walk
from re import compile
from sys import argv


PloneTestCase.setupPloneSite()

from zope.testing import doctest
OPTIONFLAGS = (doctest.REPORT_ONLY_FIRST_FAILURE |
               doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE)

class CustomerizeFunctionalTestCase(PloneTestCase.FunctionalTestCase):

    layer = layer.PloneCustomerize
    
    def afterSetUp(self):
        """ set up the tests """
        pass

    def getBrowser(self, loggedIn=False):
        """ instantiate and return a testbrowser for convenience """
        browser = Browser()
        if loggedIn:
            user = PloneTestCase.default_user
            pwd = PloneTestCase.default_password
            browser.addHeader('Authorization', 'Basic %s:%s' % (user, pwd))
        return browser

# we check argv to enable testing of explicitely named doctests
if '-t' in argv:
    pattern = compile('.*\.(txt|rst)$')
else:
    pattern = compile('^test.*\.(txt|rst)$')

def test_suite():
    suite = TestSuite()
    docs_dir = abspath(dirname(docs.__file__)) + '/'
    for path, dirs, files in walk(docs_dir):
        for name in files:
            relative = join(path, name)[len(docs_dir):]
            if not '.svn' in split(path) and pattern.search(name):
                suite.addTest(FunctionalDocFileSuite(relative,
                    optionflags=OPTIONFLAGS,
                    package=docs.__name__,
                    test_class=CustomerizeFunctionalTestCase))
    return suite



from five.customerize.interfaces import IViewTemplateContainer
from zope.interface import implements
from OFS.Folder import Folder


class ViewTemplateContainer(Folder):
    """ a local utility storing all ttw view templates provided
        by five.customerize in a folder """
    implements(IViewTemplateContainer)

    id  = 'portal_view_customizations'
    title = 'Manages view customizations'
    meta_type = 'Plone View Customizations'

    manage_options = (
        dict(label='Registrations', action='registrations.html'),
        ) + Folder.manage_options[0:1] + Folder.manage_options[2:]

    def addTemplate(self, id, template):
        """ add the given ttw view template to the container """
        self._setObject(id, template)
        return getattr(self, id)




from Products.Five.browser import BrowserView
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.component import getSiteManager
from Acquisition import aq_inner

from plone.app.customerize import registration
from five.customerize.interfaces import ITTWViewTemplate


class RegistrationsView(BrowserView):

    def getTemplateViewRegistrations(self):
        """ get all global view registrations and cycle through the local
            ones to see which views have already been customized ttw """
        regs = []
        local = {}
        for reg in self.getLocalRegistrations():
            local[(reg.required[0], str(reg.name))] = reg
        for reg in registration.templateViewRegistrations():
            lreg = local.get((reg.required[0], str(reg.name)), None)
            if lreg is not None:
                regs.append(lreg)
            else:
                regs.append(reg)
        return registration.templateViewRegistrationGroups(regs)  

    def getTemplateCodeFromRegistration(self):
        reg = self.getRegistrationFromRequest()
        return registration.getTemplateCodeFromRegistration(reg)

    def getTemplateViewRegistrationInfo(self):
        reg = self.getRegistrationFromRequest()
        return list(registration.templateViewRegistrationInfos([reg]))[0]

    def getRegistrationFromRequest(self):
        form = self.context.request.form
        return registration.findTemplateViewRegistration(form['for_name'],
            form['type_name'], form['view_name'])

    def registerTTWView(self, viewzpt, reg):
        sm = getSiteManager(self.context)
        sm.registerAdapter(viewzpt, required = reg.required,
                           provided = reg.provided, name = reg.name)

    def customizeTemplate(self):
        reg = self.getRegistrationFromRequest()
        viewzpt = registration.customizeTemplate(reg)
        self.registerTTWView(viewzpt, reg)
        path = aq_inner(viewzpt).getPhysicalPath()
        url = self.request.physicalPathToURL(path) + "/manage_workspace"
        self.request.response.redirect(url)

    def getLocalRegistrations(self):
        components = getSiteManager(self.context)
        for reg in components.registeredAdapters():
            if (len(reg.required) == 2 and
                    reg.required[1].isOrExtends(IBrowserRequest) and
                    ITTWViewTemplate.providedBy(reg.factory)):
                yield reg




from setuptools import setup, find_packages

setup(name = 'plone.app.customerize',
      version = '1.0rc1',
      description = 'Integrate five.customerize into Plone.',
      keywords = 'customerize plone five views page templates zmi',
      author = 'Plone Foundation',
      author_email = 'plone-developers@lists.sourceforge.net',
      url = 'http://svn.plone.org/svn/plone/plone.app.customerize/',
      download_url = 'http://cheeseshop.python.org/pypi/plone.app.customerize/',
      license = 'GPL',
      packages = find_packages(),
      namespace_packages = ['plone.app'],
      include_package_data = True,
      install_requires = ['setuptools',],
      platforms = 'Any',
      zip_safe = False,
      classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Plone',
        'Framework :: Zope2',
        'Intended Audience :: Other Audience',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
      ],
      long_description = """\
        This package integrates five.customerize_ into Plone, which enables
        site administrators to customize five/zope3-style views TTW via the
        ZMI in a way similar to it's possible to customize filesystem-based
        skin elements via the portal skin "customize" procedure.
        
          .. _five.customerize: http://svn.zope.org/five.customerize/ """,
)




from Products.Five.browser import BrowserView
from five.customerize.interfaces import IViewTemplateContainer, ITTWViewTemplate
from five.customerize.browser import mangleAbsoluteFilename
from five.customerize.zpt import TTWViewTemplate
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.component import getGlobalSiteManager, getUtility


def getViews(type):
    """ get all view registrations (stolen from zope.app.apidoc.presentation) """
    gsm = getGlobalSiteManager()
    for reg in gsm.registeredAdapters():
        if (len(reg.required) > 0 and
                reg.required[-1] is not None and
                reg.required[-1].isOrExtends(type)):
            for required_iface in reg.required[:-1]:
                yield reg

def interfaceName(iface):
    """ return a sensible name for the given interface """
    name = getattr(iface, '__name__', repr(iface))
    return getattr(iface, '__identifier__', name)

def templateViewRegistrations():
    for reg in getViews(IBrowserRequest):
        factory = reg.factory
        while hasattr(factory, 'factory'):
            factory = factory.factory
        # TODO: this should really be dealt with using
        # a marker interface on the view factory
        if hasattr(factory, '__name__') and \
               factory.__name__.startswith('SimpleViewClass'):
            yield reg

def templateViewRegistrationInfos(regs):
    def regkey(reg):
        return reg.name
    for reg in sorted(regs, key=regkey):
        if ITTWViewTemplate.providedBy(reg.factory):
            zptfile = None
            zcmlfile = None
            customized = reg.factory.getId()    # TODO: can we get an absolute url?
        else:
            zptfile = mangleAbsoluteFilename(reg.factory.index.filename)
            zcmlfile = mangleAbsoluteFilename(reg.info.file)
            customized = None
        yield {
            'viewname': reg.name,
            'for': interfaceName(reg.required[0]),
            'type': interfaceName(reg.required[1]),
            'zptfile': zptfile,
            'zcmlfile': zcmlfile,
            'customized': customized,
        }

def templateViewRegistrationGroups(regs):
    ifaces = {}
    for reg in templateViewRegistrationInfos(regs):
        key = reg['for']
        if ifaces.has_key(key):
            ifaces[key]['views'].append(reg)
        else:
            ifaces[key] = { 'name': key, 'views': [reg] }
    return sorted(ifaces.values(), cmp=lambda a,b: cmp(a['name'], b['name']))

def findTemplateViewRegistration(for_name, type_name, viewname):
    for reg in templateViewRegistrations():
        if interfaceName(reg.required[0]) == for_name and \
           interfaceName(reg.required[1]) == type_name and \
           reg.name == viewname:
            return reg

def generateIdFromRegistration(reg):
    return '%s-%s' % (
        interfaceName(reg.required[0]).lower(),
        reg.name
    )

def getViewClassFromRegistration(reg):
    # The view class is generally auto-generated, we usually want
    # the first base class, though if the view only has one base
    # (generally object or BrowserView) we return the full class
    # and hope that it can be pickled        
    klass = reg.factory
    base = klass.__bases__[0]
    if base is BrowserView or base is object:
        return klass
    return base

def getTemplateCodeFromRegistration(reg):
    template = reg.factory.index
    # TODO: we can't do template.read() here because of a bug in
    # Zope 3's ZPT implementation.
    return open(template.filename, 'rb').read()

def getViewPermissionFromRegistration(reg):
    permissions = reg.factory.__ac_permissions__
    for permission, methods in permissions:
        if methods[0] in ('', '__call__'):
            return permission

def createTTWViewTemplate(reg):
    return TTWViewTemplate(
        id = str(generateIdFromRegistration(reg)),
        text = getTemplateCodeFromRegistration(reg),
        view = getViewClassFromRegistration(reg),
        permission = getViewPermissionFromRegistration(reg))

def customizeTemplate(reg):
    viewzpt = createTTWViewTemplate(reg)
    container = getUtility(IViewTemplateContainer)
    return container.addTemplate(viewzpt.getId(), viewzpt)




from Products.CMFCore.utils import ToolInit
from plone.app.customerize.tool import ViewTemplateContainer


def initialize(context):
    """ initialize function called when used as a zope2 product """

    ToolInit('plone.app.customerize',
             tools = (ViewTemplateContainer,),
             icon = 'tool.gif',
    ).initialize(context)

